const Contract = require('./contract');

const Networks = {
  development: require('../build/networks/development'),
  rinkeby: require('../build/networks/rinkeby'),
  unknown: { id: '*', name: 'unknown'}
};

Networks[1] = Networks.main;
Networks[4] = Networks.rinkeby;
Networks[50] = Networks.development;
Networks[5777] = Networks.development;

class Contracts {
  constructor(network) {
    this.names = [];
    this.lookup = {};
    this.contract = {};
    this.addresses = {};
    this.network = Networks[network] || Networks.unknown;

    this.addContract('Registry');
    this.addContract('Governance');
    this.addContract('DomainToken');
    this.addContract('RegistryToken');
    this.addContract('GovernanceToken');
  }

  addContract(name) {
    if (this.contract[name]) { return false; }

    let contract = new Contract(name, this.network);
    let addr = contract.address;

    this[name] = contract;
    this.names.push(name);
    this.contract[name] = contract;

    if (addr) {
      this.lookup[addr] = contract;
      this.addresses[name] = addr;
    }

    return contract;
  }

  decodeEvent(ev) {
    if (Array.isArray(ev)) { return ev.map(this.decodeEvent.bind(this)); }
    let {data, topics, decoded, event} = ev;
    let contract = this.lookup[ev.address];

    if (contract) {
      let definition = contract.abiFor(topics[0]);
      event = definition.name;
      decoded = contract.decodeEvent(data, topics, definition);
      contract = contract.name;
    }

    return {...ev, decoded, event, contract};
  }

  decodeTransaction(tx) {
    if (typeof tx === 'string') { return tx; }
    if (Array.isArray(tx)) { return tx.map(this.decodeTransaction.bind(this)); }

    let contract = this.lookup[tx.to];

    if (contract) {
      let {name, decoded} = contract.decodeTransaction(tx.input);
      contract = contract.name;
      return {...tx, decoded, name, contract};
    } else { return tx;}
  }

  static for (network) {
    return new Contracts(network);
  }
};

Contracts.Contract = Contract;
Contracts.Networks = Networks;

module.exports = Contracts;
